=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2007-01-23 14:29:36 +0000
+++ bzrlib/bzrdir.py	2007-02-04 17:41:12 +0000
@@ -524,17 +524,19 @@
                 redirected = False
             except errors.RedirectRequested, e:
 
-                relpath = transport.relpath(e.source)
+                qualified_source = e.get_source_url()
+                relpath = transport.relpath(qualified_source)
                 if not e.target.endswith(relpath):
                     # Not redirected to a branch-format, not a branch
                     raise errors.NotBranchError(path=e.target)
 
                 target = e.target[:-len(relpath)]
-                note('%s has been%s redirected to %s',
+                note('%s is%s redirected to %s',
                      transport.base,
                      e.permanently,
                      target)
                 # Let's try with a new transport
+                qualified_target = e.get_target_url()[:-len(relpath)]
                 transport = get_transport(target)
                 redirections += 1
         if redirected:
@@ -1257,7 +1259,7 @@
 
     @classmethod
     def register_control_format(klass, format):
-        """Register a format that does not use '.bzrdir' for its control dir.
+        """Register a format that does not use '.bzr' for its control dir.
 
         TODO: This should be pulled up into a 'ControlDirFormat' base class
         which BzrDirFormat can inherit from, and renamed to register_format 
@@ -1289,10 +1291,6 @@
         klass._control_formats.remove(format)
 
 
-# register BzrDirFormat as a control format
-BzrDirFormat.register_control_format(BzrDirFormat)
-
-
 class BzrDirFormat4(BzrDirFormat):
     """Bzr dir format 4.
 
@@ -1527,9 +1525,16 @@
         except KeyError:
             raise errors.UnknownFormatError(format=format_string)
 
+    # REVIEWER: The following line is black magic for me. Any
+    # pointer appreciated
     repository_format = property(__return_repository_format, __set_repository_format)
 
 
+# register BzrDirMetaFormat1 as a control format. Our only
+# concrete control format (BzrDirFormat is an abstract one).
+BzrDirFormat.register_control_format(BzrDirMetaFormat1)
+
+# Register bzr formats
 BzrDirFormat.register_format(BzrDirFormat4())
 BzrDirFormat.register_format(BzrDirFormat5())
 BzrDirFormat.register_format(BzrDirFormat6())

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2007-01-23 14:29:36 +0000
+++ bzrlib/errors.py	2007-02-04 17:41:12 +0000
@@ -1180,7 +1180,7 @@
 
     _fmt = '%(source)s is%(permanently)s redirected to %(target)s'
 
-    def __init__(self, source, target, is_permament=False):
+    def __init__(self, source, target, is_permament=False, qual_proto=None):
         self.source = source
         self.target = target
         if is_permament:
@@ -1188,8 +1188,38 @@
         else:
             self.permanently = ''
         self.is_permament = is_permament
+        self._qualified_proto = qual_proto
         TransportError.__init__(self)
 
+    def _requalify_url(self, url):
+        """Restore the qualified proto in front of the url"""
+        # When this exception is raised, source and target are in
+        # user readable format. But some transports mau use a
+        # different proto (http+urllib:// will present http:// to
+        # the user. If a qualified proto is specified, the code
+        # trapping the exeception can get the qualified urls to
+        # properly handle the redirection themself (creating a
+        # new transport object from the target url for example).
+        if self._qualified_proto is None:
+            return url
+
+        # The TODO related to NotBranchError mention that doing
+        # that kind of manipulation on the urls may not be the
+        # exception object job. On the other hand, this object is
+        # the interface between the code and the user so
+        # presenting the urls in different ways is indeed its
+        # job...
+        import urlparse
+        proto, netloc, path, query, fragment = urlparse.urlsplit(url)
+        return urlparse.urlunsplit((self._qualified_proto, netloc, path,
+                                   query, fragment))
+
+    def get_source_url(self):
+        return self._requalify_url(self.source)
+
+    def get_target_url(self):
+        return self._requalify_url(self.source)
+
 
 class UnknownHint(TransportError):
 

=== modified file 'bzrlib/tests/HTTPTestUtil.py'
--- bzrlib/tests/HTTPTestUtil.py	2006-12-22 09:54:52 +0000
+++ bzrlib/tests/HTTPTestUtil.py	2007-02-04 17:41:12 +0000
@@ -177,7 +177,7 @@
 
 
 class TestCaseWithTwoWebservers(TestCaseWithWebserver):
-    """A support class providinf readonly urls (on two servers) that are http://.
+    """A support class providing readonly urls on two servers that are http://.
 
     We setup two webservers to allows various tests involving
     proxies or redirections from one server to the other.
@@ -225,4 +225,31 @@
         return TestingHTTPRequestHandler.translate_path(self, path)
 
 
+class RedirectRequestHandler(TestingHTTPRequestHandler):
+    """Redirect all request to the specified server"""
+
+    def parse_request(self):
+        """Redirect a single HTTP request to another host"""
+        valid = TestingHTTPRequestHandler.parse_request(self)
+        if valid:
+            # Just redirect permanently
+            self.send_response(301)
+            target = 'http://%s:%s%s' % (self.server.test_case.redirect_to_host,
+                                         self.server.test_case.redirect_to_port,
+                                         self.path)
+            self.send_header('Location', target)
+            self.end_headers()
+            return False # The job is done
+        return valid
+
+
+class HTTPServerRedirecting(HttpServer):
+    """An HttpServer redirecting to another server """
+
+    def __init__(self, redir_host, redir_port,
+                 request_handler=RedirectRequestHandler):
+        HttpServer.__init__(self, request_handler)
+        self.redirect_to_host = redir_host
+        self.redirect_to_port = redir_port
+
 

=== modified file 'bzrlib/tests/HttpServer.py'
--- bzrlib/tests/HttpServer.py	2006-12-13 17:21:18 +0000
+++ bzrlib/tests/HttpServer.py	2007-02-04 17:41:12 +0000
@@ -153,7 +153,6 @@
             self.end_headers()
             self.send_range_content(file, start, end - start + 1)
             self.wfile.write("--%s\r\n" % boundary)
-            pass
 
     def do_GET(self):
         """Serve a GET request.
@@ -267,18 +266,23 @@
     def __init__(self, request_handler=TestingHTTPRequestHandler):
         Server.__init__(self)
         self.request_handler = request_handler
+        self.host = 'localhost'
+        self.port = 0
+        self._httpd = None
 
     def _get_httpd(self):
-        return TestingHTTPServer(('localhost', 0),
-                                  self.request_handler,
-                                  self)
+        if self._httpd is None:
+            self._httpd = TestingHTTPServer((self.host, self.port),
+                                            self.request_handler,
+                                            self)
+            host, self.port = self._httpd.socket.getsockname()
+        return self._httpd
 
     def _http_start(self):
-        httpd = None
         httpd = self._get_httpd()
-        host, self.port = httpd.socket.getsockname()
-        self._http_base_url = '%s://localhost:%s/' % (self._url_protocol,
-                                                      self.port)
+        self._http_base_url = '%s://%s:%s/' % (self._url_protocol,
+                                               self.host,
+                                               self.port)
         self._http_starting.release()
         httpd.socket.settimeout(0.1)
 

=== modified file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	2006-12-15 02:47:54 +0000
+++ bzrlib/tests/test_http.py	2007-02-04 17:41:12 +0000
@@ -42,6 +42,7 @@
     BadStatusRequestHandler,
     FakeProxyRequestHandler,
     ForbiddenRequestHandler,
+    HTTPServerRedirecting,
     InvalidStatusRequestHandler,
     NoRangeRequestHandler,
     SingleRangeRequestHandler,
@@ -814,3 +815,55 @@
                         TestRanges,
                         TestCaseWithWebserver):
     """Test the Range header in GET methods for pycurl implementation"""
+
+
+class TestRedirections(object):
+    """Test redirection from the 'old' server to the 'new' server.
+
+    This MUST be used by daughter classes that also inherit from
+    TestCaseWithTwoWebservers.
+
+    We can't inherit directly from TestCaseWithTwoWebservers or the
+    test framework will try to create an instance which cannot
+    run, its implementation being incomplete. 
+    """
+
+    def create_transport_secondary_server(self):
+        """Create the secondary server redirecting to the primary server"""
+        new = self.get_new_server()
+
+        return HTTPServerRedirecting(new.host, new.port)
+
+    def get_old_server(self):
+        """The redirected server"""
+        return self.get_secondary_server()
+
+    def get_new_server(self):
+        """The redirected to server"""
+        return self.get_readonly_server()
+
+    def setUp(self):
+        super(TestRedirections, self).setUp()
+        new = self.get_new_server()
+        old = self.get_old_server()
+
+        self.build_tree_contents([('a', '0123456789')],)
+
+        self.new_transport = self._transport(new.get_url())
+        self.old_transport = self._transport(old.get_url())
+
+    def test_redirected(self):
+        old = self.old_transport
+        hints = old.create_get_hints(follow_redirections=False)
+        self.assertRaises(errors.RedirectRequested, old.get, 'a', **hints)
+
+    def test_redirection_loop(self):
+        pass
+
+
+class TestRedirections_urllib(TestRedirections,
+                              TestCaseWithTwoWebservers):
+    """Tests redirections for urllib implementation"""
+
+    _transport = HttpTransport_urllib
+

=== modified file 'bzrlib/tests/test_smart_transport.py'
--- bzrlib/tests/test_smart_transport.py	2006-11-21 01:08:52 +0000
+++ bzrlib/tests/test_smart_transport.py	2007-02-04 17:41:12 +0000
@@ -1478,8 +1478,12 @@
             'Content-Length: 6\r\n'
             '\r\n'
             'hello\n')
-        request_handler = SmartRequestHandler(
-            socket, ('localhost', 80), httpd)
+        # Beware: the ('localhost', 80) below is the
+        # client_address parameter, but we don't have one because
+        # we have defined a socket which is not bind to an
+        # address. The test framework never uses this client
+        # address, so far...
+        request_handler = SmartRequestHandler(socket, ('localhost', 80), httpd)
         response = socket.writefile.getvalue()
         self.assertStartsWith(response, 'HTTP/1.0 200 ')
         # This includes the end of the HTTP headers, and all the body.

=== modified file 'bzrlib/transport/http/__init__.py'
--- bzrlib/transport/http/__init__.py	2007-01-25 09:58:50 +0000
+++ bzrlib/transport/http/__init__.py	2007-02-04 17:41:12 +0000
@@ -143,7 +143,7 @@
             self._query, self._fragment) = urlparse.urlparse(self.base)
         self._qualified_proto = apparent_proto
         # range hint is handled dynamically throughout the life
-        # of the object. We start by trying mulri-range requests
+        # of the object. We start by trying multi-range requests
         # and if the server returns bougs results, we retry with
         # single range requests and, finally, we forget about
         # range if the server really can't understand. Once

=== modified file 'bzrlib/transport/http/_urllib.py'
--- bzrlib/transport/http/_urllib.py	2007-01-25 09:58:50 +0000
+++ bzrlib/transport/http/_urllib.py	2007-02-04 17:41:12 +0000
@@ -114,7 +114,8 @@
                 and code in (301, 302, 303, 307):
             raise errors.RedirectRequested(request.get_full_url(),
                                            request.redirected_to,
-                                           is_permament=(code == 301))
+                                           is_permament=(code == 301),
+                                           qual_proto=self._qualified_proto)
 
         if request.redirected_to is not None:
             mutter('redirected from: %s to: %s' % (request.get_full_url(),

